package be.iminds.ilabt.jfed.testing.tests.highlevel;

import be.iminds.ilabt.jfed.experiment.setup.ExperimentSetupConfig;
import be.iminds.ilabt.jfed.testing.shared.Proxy;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.dropwizard.jackson.Jackson;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.testng.annotations.Test;

import java.io.IOException;
import java.util.Map;
import java.util.regex.Pattern;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;

public class GuiLogicTestConfigTest {
    private static final Logger LOG = LoggerFactory.getLogger(GuiLogicTestConfigTest.class);
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();

    @Test
    public void testDefaults() throws Exception {
        GuiLogicTestConfig guiLogicTestConfig = new GuiLogicTestConfig();
        String jsonString = guiLogicTestConfig.stringValue();

        String prettyJsonString = MAPPER.writerWithDefaultPrettyPrinter().writeValueAsString(guiLogicTestConfig);
        System.out.println("Default GuiLogicTestConfig JSON: "+prettyJsonString+"");

        assertSameJsonObjects(jsonString, "{}");
        assertSameJsonObjects(prettyJsonString, "{}");

        checkDefaults(guiLogicTestConfig);
    }

    @Test
    public void testDefaultsRecover() throws Exception {
        String jsonString = "{}";
        GuiLogicTestConfig guiLogicTestConfig = MAPPER.readValue(jsonString, GuiLogicTestConfig.class);

        String prettyJsonString = MAPPER.writerWithDefaultPrettyPrinter().writeValueAsString(guiLogicTestConfig);
        System.out.println("Recovered Default GuiLogicTestConfig JSON: "+prettyJsonString+"");

        assertSameJsonObjects(prettyJsonString, jsonString);

        checkDefaults(guiLogicTestConfig);
    }

    private static String nonDefaultWithSingleAnsibleTestJsonString = "{\n" +
            "\t\"amConnectionProxy\": {\n" +
            "\t\t\"mode\": \"USER_AUTHORITY\"\n" +
            "\t},\n" +
            "\t\"ansibleTest\": {\n" +
            "\t\t\"enabled\": true,\n" +
            "\t\t\"debug\": true,\n" +
            "\t\t\"playbookUrl\": \"http://www.gpolab.bbn.com/experiment-support/HelloGENI/hellogeni_ansible.yml\",\n" +
            "\t\t\"playbookContent\": null,\n" +
            "\t\t\"failureRegex\": \"(failed=[^0]|unreachable=[^0])\",\n" +
            "\t\t\"successRegex\": \"ok=[^0]\"\n" +
            "\t}\n" +
            "}\n";
    private static String nonDefaultWithMultiAnsibleTestsJsonString = "{\n" +
            "\t\"amConnectionProxy\": {\n" +
            "\t\t\"mode\": \"USER_AUTHORITY\"\n" +
            "\t},\n" +
            "\t\"ansibleTests\": [{\n" +
            "\t\t\"enabled\": true,\n" +
            "\t\t\"debug\": true,\n" +
            "\t\t\"playbookUrl\": \"http://www.gpolab.bbn.com/experiment-support/HelloGENI/hellogeni_ansible.yml\",\n" +
            "\t\t\"playbookContent\": null,\n" +
            "\t\t\"failureRegex\": \"(failed=[^0]|unreachable=[^0])\",\n" +
            "\t\t\"successRegex\": \"ok=[^0]\"\n" +
            "\t}]\n" +
            "}\n";
    private static String nonDefaultMinimalJsonString = "{\n" +
            "\t\"amConnectionProxy\": {\n" +
            "\t\t\"mode\": \"USER_AUTHORITY\"\n" +
            "\t},\n" +
            "\t\"ansibleTests\": [ {\n" +
            "\t\t\"enabled\": true,\n" +
            "\t\t\"debug\": true,\n" +
            "\t\t\"playbookUrl\": \"http://www.gpolab.bbn.com/experiment-support/HelloGENI/hellogeni_ansible.yml\"\n" +
            "\t} ]\n" +
            "}\n";

    @Test
    public void testNonDefaultsRecover() throws Exception {
        GuiLogicTestConfig guiLogicTestConfig = MAPPER.readValue(nonDefaultWithSingleAnsibleTestJsonString, GuiLogicTestConfig.class);

        String prettyJsonString = MAPPER.writerWithDefaultPrettyPrinter().writeValueAsString(guiLogicTestConfig);
        System.out.println("Recovered Default GuiLogicTestConfig JSON: "+prettyJsonString+"");

        assertSameJsonObjects(prettyJsonString, nonDefaultMinimalJsonString);

        checkNonDefault(guiLogicTestConfig);
    }

    @Test
    public void testNonDefaultsRecover2() throws Exception {
        GuiLogicTestConfig guiLogicTestConfig = MAPPER.readValue(nonDefaultWithMultiAnsibleTestsJsonString, GuiLogicTestConfig.class);

        String prettyJsonString = MAPPER.writerWithDefaultPrettyPrinter().writeValueAsString(guiLogicTestConfig);
        System.out.println("Recovered Default GuiLogicTestConfig JSON: "+prettyJsonString+"");

        assertSameJsonObjects(prettyJsonString, nonDefaultMinimalJsonString);

        checkNonDefault(guiLogicTestConfig);
    }

    @Test
    public void testNonDefaultsRecoverMinimal() throws Exception {
        GuiLogicTestConfig guiLogicTestConfig = MAPPER.readValue(nonDefaultMinimalJsonString, GuiLogicTestConfig.class);

        String prettyJsonString = MAPPER.writerWithDefaultPrettyPrinter().writeValueAsString(guiLogicTestConfig);
        System.out.println("Recovered Default GuiLogicTestConfig JSON: "+prettyJsonString+"");

        assertSameJsonObjects(prettyJsonString, nonDefaultMinimalJsonString);

        checkNonDefault(guiLogicTestConfig);
    }


    private void checkNonDefault(GuiLogicTestConfig guiLogicTestConfig) {
        assertThat(guiLogicTestConfig.getAmConnectionProxy().getMode(), is(equalTo(Proxy.ProxyMode.USER_AUTHORITY)));
        assertThat(guiLogicTestConfig.getAmConnectionProxy().getHostName(), is(nullValue()));
        assertThat(guiLogicTestConfig.getAmConnectionProxy().getHostPort(), is(equalTo(22)));
        assertThat(guiLogicTestConfig.getAnsibleTests().get(0).isEnabled(), is(equalTo(true)));
        assertThat(guiLogicTestConfig.getAnsibleTests().get(0).getPlaybookUrl(), is(equalTo("http://www.gpolab.bbn.com/experiment-support/HelloGENI/hellogeni_ansible.yml")));
        assertThat(guiLogicTestConfig.getAnsibleTests().get(0).getPlaybookContent(), is(nullValue()));
        assertThat(guiLogicTestConfig.getAnsibleTests().get(0).getDebug(), is(equalTo(true)));
        assertThat(guiLogicTestConfig.getAnsibleTests().get(0).getFailureRegex(), is(equalTo("(failed=[^0]|unreachable=[^0])")));
        assertThat(guiLogicTestConfig.getAnsibleTests().get(0).getWarningRegex(), is(nullValue()));
        assertThat(guiLogicTestConfig.getAnsibleTests().get(0).getSuccessRegex(), is(equalTo("ok=[^0]")));
        assertThat(guiLogicTestConfig.getAnsibleTests().get(0).getProxy().getMode(), is(equalTo(Proxy.ProxyMode.OFF)));
        assertThat(guiLogicTestConfig.getNodeLoginTest().isEnabled(), is(equalTo(true)));
        assertThat(guiLogicTestConfig.getNodeLoginTest().getProxy().getMode(), is(equalTo(Proxy.ProxyMode.OFF)));
        assertThat(guiLogicTestConfig.getNodeLoginTest().getForceIpVersion(), is(nullValue()));

        assertThat(guiLogicTestConfig.getResources(), hasSize(1));
        GuiLogicTestConfig.Resource resource = guiLogicTestConfig.getResources().get(0);
//        assertThat(resource.getProvision().getProvisionExpireTimeMin(), is(equalTo(25l)));
        assertThat(((ExperimentSetupConfig.GenerateUsingJFedRequestRSpec)resource.getRequestRSpec()).getjFedResourceClass(), is(nullValue()));
        assertThat(resource.getRequestRSpec().getSource(), is(equalTo(ExperimentSetupConfig.RequestRSpecSource.GENERATE_USING_JFED)));
        assertThat(resource.getRequestRSpec().getUseNitosLease(), is(equalTo(false)));
        assertThat(resource.getSlice().getFailIfNoProject(), is(equalTo(false)));
        assertThat(resource.getSlice().getProject(), is(nullValue()));
        assertThat(resource.getSkipFinalManifestCheck(), is(equalTo(false)));
        assertThat(resource.getWaitForReady().getExtraWaitTimeSeconds(), is(equalTo(0l)));
    }

    private void checkDefaults(GuiLogicTestConfig guiLogicTestConfig) {
        assertThat(guiLogicTestConfig.getAmConnectionProxy().getMode(), is(equalTo(Proxy.ProxyMode.OFF)));
        assertThat(guiLogicTestConfig.getAmConnectionProxy().getHostName(), is(nullValue()));
        assertThat(guiLogicTestConfig.getAmConnectionProxy().getHostPort(), is(equalTo(22)));
        assertThat(guiLogicTestConfig.getAnsibleTests().get(0).isEnabled(), is(equalTo(false)));
        assertThat(guiLogicTestConfig.getAnsibleTests().get(0).getDebug(), is(equalTo(false)));
        assertThat(guiLogicTestConfig.getNodeLoginTest().isEnabled(), is(equalTo(true)));
        assertThat(guiLogicTestConfig.getNodeLoginTest().getProxy().getMode(), is(equalTo(Proxy.ProxyMode.OFF)));
        assertThat(guiLogicTestConfig.getNodeLoginTest().getForceIpVersion(), is(nullValue()));

        assertThat(guiLogicTestConfig.getResources(), hasSize(1));
        GuiLogicTestConfig.Resource resource = guiLogicTestConfig.getResources().get(0);
//        assertThat(resource.getProvision().getProvisionExpireTimeMin(), is(equalTo(25l)));
        assertThat(((ExperimentSetupConfig.GenerateUsingJFedRequestRSpec)resource.getRequestRSpec()).getjFedResourceClass(), is(nullValue()));
        assertThat(resource.getRequestRSpec().getSource(), is(equalTo(ExperimentSetupConfig.RequestRSpecSource.GENERATE_USING_JFED)));
        assertThat(resource.getRequestRSpec().getUseNitosLease(), is(equalTo(false)));
        assertThat(resource.getSlice().getFailIfNoProject(), is(equalTo(false)));
        assertThat(resource.getSlice().getProject(), is(nullValue()));
        assertThat(resource.getSkipFinalManifestCheck(), is(equalTo(false)));
        assertThat(resource.getWaitForReady().getExtraWaitTimeSeconds(), is(equalTo(0l)));
    }

    private void assertSameJsonObjects(String a, String b) throws IOException {
        Map am = MAPPER.readValue(a, Map.class);
        Map bm = MAPPER.readValue(b, Map.class);
        assertThat("json differs: "+a+"\n\n-----------\n\n"+b, am, is(equalTo(bm)));
    }

    @Test
    private void testRequestRSpecA() throws Exception {
        ExperimentSetupConfig.RequestRSpec orig =
                new ExperimentSetupConfig.GenerateUsingAdvertisementRequestRSpec();

        testRequestRSpecReconstructable(orig);
    }

    @Test
    private void testRequestRSpecB() throws Exception {
        ExperimentSetupConfig.RequestRSpec orig =
                new ExperimentSetupConfig.GenerateUsingJFedRequestRSpec();

        testRequestRSpecReconstructable(orig);
    }

    @Test
    private void testRequestRSpecC() throws Exception {
        ExperimentSetupConfig.RequestRSpec orig =
                new ExperimentSetupConfig.GenerateSimpleRequestRSpec();

        testRequestRSpecReconstructable(orig);
    }

    @Test
    private void testRequestRSpecD1() throws Exception {
        ExperimentSetupConfig.RequestRSpec orig =
                new ExperimentSetupConfig.ProvidedUrlRequestRSpec();

        testRequestRSpecReconstructable(orig);
    }

    @Test
    private void testRequestRSpecD2() throws Exception {
        ExperimentSetupConfig.RequestRSpec orig =
                new ExperimentSetupConfig.ProvidedContentRequestRSpec();

        testRequestRSpecReconstructable(orig);
    }

    @Test
    private void testRequestRSpecD3() throws Exception {
        ExperimentSetupConfig.RequestRSpec orig =
                new ExperimentSetupConfig.ProvidedFileRequestRSpec(
                        ExperimentSetupConfig.RequestRSpecSource.PROVIDE_FILE,
                        "testfile",
                        true,
                        null,
                        true,
                        true);

        testRequestRSpecReconstructable(orig);
    }

    @Test
    private void testRequestRSpecE() throws Exception {
        ExperimentSetupConfig.RequestRSpec orig =
                new ExperimentSetupConfig.AlreadyProvisionedRequestRSpec();

        testRequestRSpecReconstructable(orig);
    }

    private void testRequestRSpecReconstructable(ExperimentSetupConfig.RequestRSpec orig) throws Exception {
        Map m = MAPPER.convertValue(orig, Map.class);
        LOG.debug("orig="+orig);
        LOG.debug("map="+m);
        assert(m.containsKey("source"));
        assertThat(m.get("source"), is(equalTo(orig.getSource().name())));

        ExperimentSetupConfig.RequestRSpec reconstructed =
                MAPPER.convertValue(m, ExperimentSetupConfig.RequestRSpec.class);
        LOG.debug("reconstructed="+reconstructed);

        assertThat(reconstructed.getClass(), is(equalTo(orig.getClass())));
        assertThat(reconstructed.getSource(), is(equalTo(orig.getSource())));
        assertThat(reconstructed, is(equalTo(orig)));
    }

    @Test
    public void regexDemo() {
        assertThat(Pattern.compile("failed=[^0]").matcher("failed=0").find(), is(false));
        assertThat(Pattern.compile("failed=[^0]").matcher("failed=5").find(), is(true));
        assertThat(Pattern.compile("failed=[^0]").matcher("failed=3").find(), is(true));
        assertThat(Pattern.compile("failed=[^0]").matcher("blahblah=6").find(), is(false));
        assertThat(Pattern.compile("failed=[^0]").matcher("blaherrorblah").find(), is(false));
        assertThat(Pattern.compile("failed=[^0]").matcher("fail=5").find(), is(false));

        //"failureRegex" : "(failed=[^0])|(error)"
        assertThat(Pattern.compile("(failed=[^0])|(error)").matcher("failed=0").find(), is(false));
        assertThat(Pattern.compile("(failed=[^0])|(error)").matcher("failed=5").find(), is(true));
        assertThat(Pattern.compile("(failed=[^0])|(error)").matcher("failed=3").find(), is(true));
        assertThat(Pattern.compile("(failed=[^0])|(error)").matcher("blahblah=6").find(), is(false));
        assertThat(Pattern.compile("(failed=[^0])|(error)").matcher("blaherrorblah").find(), is(true));
        assertThat(Pattern.compile("(failed=[^0])|(error)").matcher("fail=5").find(), is(false));

        assertThat(Pattern.compile("(failed=[^0]|unreachable=[^0])").matcher("failed=0").find(), is(false));
        assertThat(Pattern.compile("(failed=[^0]|unreachable=[^0])").matcher("failed=5").find(), is(true));
        assertThat(Pattern.compile("(failed=[^0]|unreachable=[^0])").matcher("failed=3").find(), is(true));
        assertThat(Pattern.compile("(failed=[^0]|unreachable=[^0])").matcher("blahblah=6").find(), is(false));
        assertThat(Pattern.compile("(failed=[^0]|unreachable=[^0])").matcher("blaherrorblah").find(), is(false));
        assertThat(Pattern.compile("(failed=[^0]|unreachable=[^0])").matcher("fail=5").find(), is(false));
        assertThat(Pattern.compile("(failed=[^0]|unreachable=[^0])").matcher("unreachable=0").find(), is(false));
        assertThat(Pattern.compile("(failed=[^0]|unreachable=[^0])").matcher("unreachable=5").find(), is(true));
        assertThat(Pattern.compile("(failed=[^0]|unreachable=[^0])").matcher("unreachable=3").find(), is(true));
        assertThat(Pattern.compile("(failed=[^0]|unreachable=[^0])").matcher("blahblah=6").find(), is(false));
        assertThat(Pattern.compile("(failed=[^0]|unreachable=[^0])").matcher("blaherrorblah").find(), is(false));
        assertThat(Pattern.compile("(failed=[^0]|unreachable=[^0])").matcher("unreach=5").find(), is(false));
    }
}
